'use strict';function _interopRequireDefault(obj) {return obj && obj.__esModule ? obj : { 'default': obj };}var _chai = require(


'chai');var _mainDataBam = require(

'../../main/data/bam');var _mainDataBam2 = _interopRequireDefault(_mainDataBam);var _mainSourcesBamDataSource = require(
'../../main/sources/BamDataSource');var _mainSourcesBamDataSource2 = _interopRequireDefault(_mainSourcesBamDataSource);var _mainContigInterval = require(
'../../main/ContigInterval');var _mainContigInterval2 = _interopRequireDefault(_mainContigInterval);var _MappedRemoteFile = require(
'../MappedRemoteFile');var _MappedRemoteFile2 = _interopRequireDefault(_MappedRemoteFile);

describe('BamDataSource', function () {
  function getTestSource() {
    // See test/data/README.md for provenance of these files.
    var remoteBAI = new _MappedRemoteFile2['default']('/test-data/dream.synth3.bam.bai.mapped', 
    [[8054040, 8242920]]);

    var remoteBAM = new _MappedRemoteFile2['default']('/test-data/dream.synth3.bam.mapped', 
    [[0, 69453], [163622109888, 163622739903]]);

    var bam = new _mainDataBam2['default'](remoteBAM, remoteBAI, { 
      // "chunks" is usually an array; here we take advantage of the
      // Object-like nature of JavaScript arrays to create a sparse array.
      "chunks": { "19": [8054040, 8242920] }, 
      "minBlockIndex": 69454 });


    return _mainSourcesBamDataSource2['default'].createFromBamFile(bam);}


  it('should extract features in a range', function (done) {
    this.timeout(5000);
    var source = getTestSource();

    // This range matches the "large, dense" test in bam-test.js
    var range = new _mainContigInterval2['default']('20', 31511349, 31514172);
    var reads = source.getFeaturesInRange(range);
    (0, _chai.expect)(reads).to.deep.equal([]);

    source.on('newdata', function () {
      var reads = source.getFeaturesInRange(range);
      (0, _chai.expect)(reads).to.have.length(1112);
      (0, _chai.expect)(reads[0].toString()).to.equal('20:31511251-31511351');
      (0, _chai.expect)(reads[1111].toString()).to.equal('20:31514171-31514271');
      done();});

    source.rangeChanged({ 
      contig: range.contig, 
      start: range.start(), 
      stop: range.stop() });});



  it('should fetch nearby features', function (done) {
    this.timeout(5000);
    var source = getTestSource();

    // Requests are for 'chr20', while the canonical name is just '20'.
    var range = new _mainContigInterval2['default']('chr20', 31512050, 31512150);

    var rangeBefore = new _mainContigInterval2['default']('chr20', 31512000, 31512050);

    var rangeAfter = new _mainContigInterval2['default']('chr20', 31512150, 31512199);

    var reads = source.getFeaturesInRange(range);
    (0, _chai.expect)(reads).to.deep.equal([]);

    var networkEvents = [];
    source.on('networkprogress', function (event) {networkEvents.push(event);});
    source.on('networkdone', function () {networkEvents.push('networkdone');});

    // Fetching [50, 150] should cache [0, 200]
    source.on('newdata', function () {
      var reads = source.getFeaturesInRange(range);
      (0, _chai.expect)(reads).to.have.length(18);
      (0, _chai.expect)(reads[0].toString()).to.equal('20:31511951-31512051');
      (0, _chai.expect)(reads[17].toString()).to.equal('20:31512146-31512246');

      var readsBefore = source.getFeaturesInRange(rangeBefore);

      var readsAfter = source.getFeaturesInRange(rangeAfter);

      (0, _chai.expect)(readsBefore).to.have.length(26);
      (0, _chai.expect)(readsAfter).to.have.length(12);

      (0, _chai.expect)(networkEvents).to.deep.equal([
      // TODO: figure out why this notification is getting dropped.
      // {'status': 'Fetching BAM header'},
      { 'status': 'Fetching BAM index' }, 
      { 'numRequests': 1 }, 
      { 'numRequests': 2 }, 
      { 'numRequests': 3 }, 
      { 'numRequests': 4 }, 
      'networkdone']);


      // TODO: test that fetching readsBefore and readsAfter produces no
      // new network fetches.
      done();});

    source.rangeChanged({ 
      contig: range.contig, 
      start: range.start(), 
      stop: range.stop() });});



  it('should only fetch new features', function (done) {
    var source = getTestSource();
    source.once('newdata', function (range) {
      (0, _chai.expect)(range.toString()).to.equal('20:31512100-31512400'); // expanded range
      source.once('newdata', function (range) {
        (0, _chai.expect)(range.toString()).to.equal('20:31512000-31512099'); // only 100bp
        done();});

      // This range is 100bp to the left of the previous one.
      source.rangeChanged({ contig: '20', start: 31512050, stop: 31512250 });});

    source.rangeChanged({ contig: '20', start: 31512150, stop: 31512350 });});});