set -ex

# make sure docker is installed. If not, throw error
docker -v
rc=$?; if [[ $rc != 0 ]]; then
        echo "Docker is not installed. Have you run install.sh?"
        exit $rc
fi

# Split args into Spark and notebook args
DD=False  # DD is "double dash"
PRE_DD=()
POST_DD=()

# by default, runs mango browser (mango-submit)
for ARG in "$@"; do
 shift
 if [[ $ARG == "--" ]]; then
   DD=True
   POST_DD=( "$@" )
   break
 fi
 PRE_DD+=("$ARG")
done

PRE_DD_ARGS="${PRE_DD[@]}"
POST_DD_ARGS="${POST_DD[@]}"

# get location of genome file
ARRAY=($POST_DD_ARGS)
GENOME_FILE_LOCATION=${ARRAY[0]}

# if genome file is no the host, then mount it for docker to access
if [ -f "$GENOME_FILE_LOCATION" ]; then
    GENOME_FILE_MNT="-v ${GENOME_FILE_LOCATION}:${GENOME_FILE_LOCATION}"
fi

export SPARK_HOME=/usr/lib/spark
export SPARK_CONF_DIR=/usr/lib/spark/conf
export HADOOP_HOME=/usr/lib/hadoop
export HADOOP_LIBEXEC_DIR=$HADOOP_HOME/libexec
export HADOOP_CONF_DIR=$HADOOP_HOME/etc/hadoop
export HADOOP_HDFS=/usr/lib/hadoop-hdfs
export HADOOP_YARN=/usr/lib/hadoop-yarn
export HADOOP_MAPREDUCE=/usr/lib/hadoop-mapreduce
export HADOOP_LZO=/usr/lib/hadoop-lzo/lib

# ENV Variables for EMR/AWS
export AWS_SDK=/usr/share/aws

# get UUID name for docker container
uuid=$(uuidgen)
DOCKER_CONTAINER_NAME=mango_browser_${uuid}

# s3a commands
EXTRA_CLASSPATH=/usr/lib/hadoop/hadoop-aws*:${AWS_SDK}/*:/usr/lib/hadoop-lzo/lib/*

SPARK_LIBRARY_PATH=${SPARK_LIBRARY_PATH}:${HADOOP_LZO}/native


sudo docker run \
 --net=host \
      ${GENOME_FILE_MNT} \
      -v ${SPARK_HOME}:${SPARK_HOME} \
      -v ${SPARK_CONF_DIR}:${SPARK_CONF_DIR} \
      -v ${HADOOP_HOME}:${HADOOP_HOME} \
      -v ${HADOOP_HDFS}:${HADOOP_HDFS} \
      -v ${HADOOP_YARN}:${HADOOP_YARN} \
      -v ${AWS_SDK}:${AWS_SDK} \
      -v /root/.ivy2:/root/.ivy2  \
      -v ${HADOOP_LZO}:${HADOOP_LZO} \
      -v ${HADOOP_CONF_DIR}:${HADOOP_CONF_DIR} \
      -v ${HADOOP_MAPREDUCE}:${HADOOP_MAPREDUCE} \
      -e SPARK_CLASSPATH=${SPARK_CLASSPATH} \
      -e SPARK_HOME=${SPARK_HOME} \
      -e HADOOP_HOME=${HADOOP_HOME} \
      -e SPARK_CONF_DIR=${SPARK_CONF_DIR} \
      -e HADOOP_CONF_DIR=${HADOOP_CONF_DIR} \
      -e SPARK_DIST_CLASSPATH="/usr/lib/hadoop/etc/hadoop:/usr/lib/hadoop/lib/*:/usr/lib/hadoop/.//*:/usr/lib/hadoop-hdfs/./:/usr/lib/hadoop-hdfs/lib/*:/usr/lib/hadoop-hdfs/.//*:/usr/lib/hadoop-yarn/lib/*:/usr/lib/hadoop-yarn/.//*:/usr/lib/hadoop-mapreduce/lib/*:/usr/lib/hadoop-mapreduce/.//*" \
      -e SPARK_LIBRARY_PATH=${SPARK_LIBRARY_PATH} \
      -p 8081:8081 \
      -i \
      -t \
       quay.io/ucsc_cgl/mango:latest \
       --master yarn \
       --conf spark.hadoop.fs.s3a.impl=org.apache.hadoop.fs.s3a.S3AFileSystem \
       --conf fs.s3a.connection.maximum=50000 \
       --conf spark.kryoserializer.buffer.max=2040 \
       --packages net.fnothaft:jsr203-s3a:0.0.2 \
       --conf spark.hadoop.hadoopbam.bam.enable-bai-splitter=true \
       --conf spark.driver.extraClassPath=${EXTRA_CLASSPATH} \
       --conf spark.executor.extraClassPath=${EXTRA_CLASSPATH} \
       $PRE_DD_ARGS \
       --  \
       -port 8081 \
       $POST_DD_ARGS
